package com.tzld.piaoquan.api.service.contentplatform.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.api.common.enums.ExceptionEnum;
import com.tzld.piaoquan.api.common.enums.contentplatform.ContentPlatformGzhPlanTypeEnum;
import com.tzld.piaoquan.api.common.enums.contentplatform.PublishStageEnum;
import com.tzld.piaoquan.api.common.enums.contentplatform.QwPlanTypeEnum;
import com.tzld.piaoquan.api.common.exception.CommonException;
import com.tzld.piaoquan.api.component.*;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.*;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ext.ContentPlatformPlanMapperExt;
import com.tzld.piaoquan.api.model.bo.AdPutFlowRecordTencent;
import com.tzld.piaoquan.api.model.bo.BucketDataParam;
import com.tzld.piaoquan.api.model.bo.SmallPageUrlDetail;
import com.tzld.piaoquan.api.model.config.LoginUserContext;
import com.tzld.piaoquan.api.model.param.contentplatform.*;
import com.tzld.piaoquan.api.model.po.contentplatform.*;
import com.tzld.piaoquan.api.model.vo.GhDetailVo;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanVideoContentItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.QwPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.VideoContentItemVO;
import com.tzld.piaoquan.api.service.CgiReplyService;
import com.tzld.piaoquan.api.service.GhDetailService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformAccountService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformCooperateAccountService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformPlanService;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.growth.common.common.enums.StrategyStatusEnum;
import com.tzld.piaoquan.growth.common.dao.mapper.ext.CgiReplyBucketDataMapperExt;
import com.tzld.piaoquan.growth.common.model.bo.VideoDetail;
import com.tzld.piaoquan.growth.common.model.po.CgiReplyBucketData;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.service.MessageAttachmentService;
import com.tzld.piaoquan.growth.common.utils.MessageUtil;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ContentPlatformPlanServiceImpl implements ContentPlatformPlanService {

    @Autowired
    private ContentPlatformGzhPlanMapper gzhPlanMapper;
    @Autowired
    private ContentPlatformPlanMapperExt planMapperExt;
    @Autowired
    private ContentPlatformGzhPlanVideoMapper gzhPlanVideoMapper;
    @Autowired
    private ContentPlatformQwPlanMapper qwPlanMapper;
    @Autowired
    private ContentPlatformQwPlanVideoMapper qwPlanVideoMapper;
    @Autowired
    private CgiReplyBucketDataMapperExt cgiReplyBucketDataMapperExt;
    @Autowired
    private ContentPlatformVideoMapper contentPlatformVideoMapper;
    @Autowired
    private ContentPlatformGzhPlanChangeLogMapper gzhPlanChangeLogMapper;
    @Autowired
    private ContentPlatformCooperateAccountService cooperateAccountService;
    @Autowired
    private GhDetailService ghDetailService;
    @Autowired
    private AigcApiService aigcApiService;
    @Autowired
    private ManagerApiService managerApiService;
    @Autowired
    private MessageAttachmentService messageAttachmentService;
    @Autowired
    private VlogApiService vlogApiService;
    @Autowired
    private ContentPlatformAccountService accountService;
    @Autowired
    private AdApiService adApiService;
    @Autowired
    private CgiReplyService cgiReplyService;
    @Autowired
    private TouLiuHttpClient touLiuHttpClient;


    @Value("${vlog.share.appType:11}")
    private String shareAppType;

    @Value("${video.min.score:0.6}")
    private Double videoMinScore;

    @Value("${small_page_url}")
    private String GET_SMALL_PAGE_URL;

    @Override
    public Page<GzhPlanItemVO> gzhPlanList(GzhPlanListParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        Page<GzhPlanItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = planMapperExt.getGzhPlanCount(param, loginAccount.getId());
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformGzhPlan> planList = planMapperExt.getGzhPlanList(param,
                loginAccount.getId(), offset, param.getPageSize());
        List<GzhPlanItemVO> list = buildGzhPlanItemVOList(planList);
        result.setObjs(list);
        return result;
    }

    private List<GzhPlanItemVO> buildGzhPlanItemVOList(List<ContentPlatformGzhPlan> planList) {
        if (CollectionUtils.isEmpty(planList)) {
            return null;
        }
        List<Long> accountIds = planList.stream().map(ContentPlatformGzhPlan::getAccountId).collect(Collectors.toList());
        List<ContentPlatformGzhAccount> accountList = cooperateAccountService.getAccountListByIds(accountIds);
        Map<Long, ContentPlatformGzhAccount> accountMap = accountList.stream().collect(Collectors.toMap(ContentPlatformGzhAccount::getId,
                Function.identity()));
        List<Long> planIds = planList.stream().map(ContentPlatformGzhPlan::getId).collect(Collectors.toList());
        List<ContentPlatformGzhPlanVideo> videoList = getGzhPlanVideoList(planIds);
        Map<Long, List<ContentPlatformGzhPlanVideo>> videoMap = videoList.stream()
                .collect(Collectors.groupingBy(ContentPlatformGzhPlanVideo::getPlanId));
        List<String> ghIds = accountList.stream().map(ContentPlatformGzhAccount::getGhId).collect(Collectors.toList());
        List<CgiReplyBucketData> bucketDataList = cgiReplyService.getCgiReplyBucketDataListByGhIds(ghIds, "manual");
        Map<String, Map<Long, CgiReplyBucketData>> bucketDataMap = bucketDataList.stream()
                .collect(Collectors.groupingBy(CgiReplyBucketData::getGhId,
                        Collectors.toMap(CgiReplyBucketData::getMiniVideoId, Function.identity())));
        List<GzhPlanItemVO> result = new ArrayList<>();
        for (ContentPlatformGzhPlan gzhPlan : planList) {
            GzhPlanItemVO planItemVO = new GzhPlanItemVO();
            planItemVO.setId(gzhPlan.getId());
            planItemVO.setAccountId(gzhPlan.getAccountId());
            ContentPlatformGzhAccount gzhAccount = accountMap.get(gzhPlan.getAccountId());
            if (Objects.nonNull(gzhAccount)) {
                planItemVO.setAccountName(gzhAccount.getName());
            }
            planItemVO.setType(gzhPlan.getType());
            planItemVO.setScene(gzhPlan.getScene());
            planItemVO.setPublishStage(gzhPlan.getPublishStage());
            planItemVO.setSelectVideoType(gzhPlan.getSelectVideoType());
            List<ContentPlatformGzhPlanVideo> videoListByPlanId = videoMap.get(gzhPlan.getId());
            if (CollectionUtils.isNotEmpty(videoListByPlanId)) {
                planItemVO.setVideoCount(videoListByPlanId.size());
                List<String> titleList = videoListByPlanId.stream().map(ContentPlatformPlanService::getVideoTitle)
                        .collect(Collectors.toList());
                planItemVO.setTitle(titleList);
                List<GzhPlanVideoContentItemVO> videoVOList = new ArrayList<>();
                Map<Long, CgiReplyBucketData> bucketDataMapByGhId = bucketDataMap.get(gzhAccount.getGhId());
                for (ContentPlatformGzhPlanVideo video : videoListByPlanId) {
                    GzhPlanVideoContentItemVO videoItemVO = new GzhPlanVideoContentItemVO();
                    videoItemVO.setVideoId(video.getVideoId());
                    videoItemVO.setTitle(video.getTitle());
                    videoItemVO.setCustomTitle(video.getCustomTitle());
                    videoItemVO.setCover(video.getCover());
                    videoItemVO.setCustomCover(video.getCustomCover());
                    videoItemVO.setCustomCoverType(video.getCustomCoverType());
                    videoItemVO.setVideo(video.getVideo());
                    videoItemVO.setPageUrl(video.getPageUrl());
                    if (Objects.nonNull(bucketDataMapByGhId)) {
                        CgiReplyBucketData bucketData = bucketDataMapByGhId.get(video.getVideoId());
                        if (Objects.nonNull(bucketData)) {
                            videoItemVO.setPageUrl(bucketData.getMiniPagePath());
                        }
                    }
                    videoVOList.add(videoItemVO);
                }
                planItemVO.setVideoList(videoVOList);
            }
            planItemVO.setCreateTimestamp(gzhPlan.getCreateTimestamp());
            result.add(planItemVO);
        }
        return result;
    }

    @Override
    public List<ContentPlatformGzhPlanVideo> getGzhPlanVideoList(List<Long> planIds) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    @Override
    public void gzhPlanDelete(Long id) {
        ContentPlatformGzhPlan plan = gzhPlanMapper.selectByPrimaryKey(id);
        if (Objects.isNull(plan)) {
            throw new CommonException(ExceptionEnum.GZH_PLAN_NOT_EXISTS);
        }
        plan.setStatus(0);
        plan.setUpdateTimestamp(System.currentTimeMillis());
        gzhPlanMapper.updateByPrimaryKeySelective(plan);
        if (PublishStageEnum.PLATFORM.getVal() == plan.getPublishStage()) {
            // 调用aigc删除发布计划
            aigcApiService.changePublishPlanStatus(plan.getExternalId(), 0);
        }
    }


    @Override
    public void gzhUpdateStagePublishStatus(Long id, Integer stagePublishStatus) {
        ContentPlatformGzhPlan plan = gzhPlanMapper.selectByPrimaryKey(id);
        if (Objects.isNull(plan)) {
            throw new CommonException(ExceptionEnum.GZH_PLAN_NOT_EXISTS);
        }
        if (!Objects.equals(LoginUserContext.getUser().getId(), plan.getCreateAccountId())) {
            throw new CommonException(ExceptionEnum.PARAM_ERROR);
        }
        plan.setStagePublishStatus(stagePublishStatus);
        plan.setUpdateTimestamp(System.currentTimeMillis());
        gzhPlanMapper.updateByPrimaryKeySelective(plan);
        ContentPlatformGzhAccount gzhAccount = cooperateAccountService.getById(plan.getAccountId());
        if (StringUtils.hasText(plan.getExternalId()) && StringUtils.hasText(gzhAccount.getExternalId())) {
            aigcApiService.changePublishPlanAccountStatus(plan.getExternalId(), gzhAccount.getExternalId(), stagePublishStatus);
        }
    }

    @Override
    public void qwPlanDelete(Long id) {
        ContentPlatformQwPlan plan = qwPlanMapper.selectByPrimaryKey(id);
        if (Objects.isNull(plan)) {
            throw new CommonException(ExceptionEnum.QW_PLAN_NOT_EXISTS);
        }
        plan.setStatus(0);
        plan.setUpdateTimestamp(System.currentTimeMillis());
        qwPlanMapper.updateByPrimaryKeySelective(plan);
    }

    @Override
    public void gzhPlanSave(GzhPlanSaveParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        List<ContentPlatformGzhPlan> planExistList = planMapperExt.getGzhPlanExists(param.getAccountId(),
                param.getType(), param.getScene(), param.getId());
        if (CollectionUtils.isNotEmpty(planExistList) && param.getType() != ContentPlatformGzhPlanTypeEnum.FWH_PUSH.getVal()) {
            throw new CommonException(ExceptionEnum.GZH_SCENE_PLAN_EXISTS);
        }
        if (param.getVideoList().size() > 3) {
            throw new CommonException(ExceptionEnum.GZH_PLAN_VIDEO_SIZE_EXCEED);
        }
        ContentPlatformGzhAccount account = cooperateAccountService.getById(param.getAccountId());
        if (Objects.isNull(account)) {
            throw new CommonException(ExceptionEnum.GZH_ACCOUNT_NOT_EXISTS);
        }
        if (!StringUtils.hasText(account.getExternalId()) && param.getPublishStage().equals(PublishStageEnum.PLATFORM.getVal())) {
            throw new CommonException(ExceptionEnum.GZH_ACCOUNT_NOT_AUTH);
        }
        Long now = System.currentTimeMillis();
        ContentPlatformGzhPlan gzhPlan = new ContentPlatformGzhPlan();
        gzhPlan.setAccountId(param.getAccountId());
        gzhPlan.setType(param.getType());
        gzhPlan.setScene(param.getScene());
        gzhPlan.setPublishStage(param.getPublishStage());
        gzhPlan.setSelectVideoType(param.getSelectVideoType());
        gzhPlan.setUpdateTimestamp(now);
        List<Long> videoIds = param.getVideoList().stream().map(GzhPlanVideoContentItemParam::getVideoId).collect(Collectors.toList());
        if (param.getType() == ContentPlatformGzhPlanTypeEnum.AUTO_REPLY.getVal()) {
            // 更新gh_detail
            updateGhDetail(account, param.getSelectVideoType(), videoIds);
            // 更新cgi_reply_bucket_data
            updateCgiReplyBucketData(account.getGhId(), param.getVideoList());
        }
        if (Objects.isNull(param.getId())) {
            // 调用aigc创建发布计划
            if (PublishStageEnum.PLATFORM.getVal() == param.getPublishStage()) {
                String externalId = aigcApiService.createPublishPlan(account.getExternalId(), account.getName(), loginAccount.getChannel());
                gzhPlan.setStagePublishStatus(1);
                gzhPlan.setExternalId(externalId);
            }
            gzhPlan.setCreateAccountId(loginAccount.getId());
            gzhPlan.setCreateTimestamp(now);
            planMapperExt.insertGzhPlanReturnId(gzhPlan);
        } else {
            ContentPlatformGzhPlan oldPlan = gzhPlanMapper.selectByPrimaryKey(param.getId());
            if (PublishStageEnum.USER.getVal() == param.getPublishStage()) {
                if (StringUtils.hasText(oldPlan.getExternalId())) {
                    gzhPlan.setStagePublishStatus(0);
                    aigcApiService.changePublishPlanStatus(oldPlan.getExternalId(), 0);
                }
            } else {
                if (StringUtils.hasText(oldPlan.getExternalId())) {
                    gzhPlan.setStagePublishStatus(1);
                    aigcApiService.changePublishPlanStatus(oldPlan.getExternalId(), 1);
                } else {
                    String externalId = aigcApiService.createPublishPlan(account.getExternalId(), account.getName(), loginAccount.getChannel());
                    gzhPlan.setStagePublishStatus(1);
                    gzhPlan.setExternalId(externalId);
                }
            }
            gzhPlan.setId(param.getId());
            gzhPlanMapper.updateByPrimaryKeySelective(gzhPlan);
        }
        // 保存视频内容
        saveGzhPlanVideo(param, videoIds, gzhPlan.getId(), account, loginAccount);
        // 调用aigc重新拉取视频
        if (param.getType() == ContentPlatformGzhPlanTypeEnum.AUTO_REPLY.getVal()) {
            if (PublishStageEnum.PLATFORM.getVal() == param.getPublishStage()) {
                aigcApiService.refreshGzhAutoReplyMsgData(account.getGhId());
            } else {
                BucketDataParam bucketDataParam = new BucketDataParam();
                bucketDataParam.setGhId(account.getGhId());
                cgiReplyService.getRgiReplyData(bucketDataParam);
            }
        }
    }

    private void updateCgiReplyBucketData(String ghId, List<GzhPlanVideoContentItemParam> videoList) {
        for (GzhPlanVideoContentItemParam video : videoList) {
            if (!StringUtils.hasText(video.getCustomCover()) && !StringUtils.hasText(video.getCustomTitle())) {
                continue;
            }
            List<CgiReplyBucketData> dataList = cgiReplyService.getCgiReplyBucketDataListByVideoId(video.getVideoId());
            if (CollectionUtils.isEmpty(dataList)) {
                continue;
            }
            String existsCover = dataList.get(0).getCoverUrl();
            String coverSuffix = existsCover.substring(existsCover.indexOf("?"));
            cgiReplyBucketDataMapperExt.updateBucketDataTitleCoverByGhId(ghId, video.getVideoId(),
                    video.getCustomTitle(), video.getCustomCover() + coverSuffix);
        }
    }

    private void saveGzhPlanVideo(GzhPlanSaveParam param, List<Long> videoIds, Long planId,
                                  ContentPlatformGzhAccount account, ContentPlatformAccount loginAccount) {
        List<ContentPlatformGzhPlanVideo> existsVideo = getGzhVideoByPlanId(planId);
        List<Long> existsVideoIds = existsVideo.stream().map(ContentPlatformGzhPlanVideo::getVideoId).collect(Collectors.toList());
        Map<Long, ContentPlatformGzhPlanVideo> existsVideoMap = existsVideo.stream()
                .collect(Collectors.toMap(ContentPlatformGzhPlanVideo::getVideoId, item -> item));
        for (ContentPlatformGzhPlanVideo item : existsVideo) {
            if (!videoIds.contains(item.getVideoId())) {
                gzhPlanVideoMapper.deleteByPrimaryKey(item.getId());
            }
        }
        for (GzhPlanVideoContentItemParam vo : param.getVideoList()) {
            if (existsVideoIds.contains(vo.getVideoId())) {
                ContentPlatformGzhPlanVideo item = existsVideoMap.get(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setCustomTitle(vo.getCustomTitle());
                item.setCover(vo.getCover());
                item.setCustomCover(vo.getCustomCover());
                item.setCustomCoverType(vo.getCustomCoverType());
                gzhPlanVideoMapper.updateByPrimaryKey(item);
            } else {
                ContentPlatformGzhPlanVideo item = new ContentPlatformGzhPlanVideo();
                item.setPlanId(planId);
                item.setVideoId(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setCustomTitle(vo.getCustomTitle());
                item.setCover(vo.getCover());
                item.setCustomCover(vo.getCustomCover());
                item.setCustomCoverType(vo.getCustomCoverType());
                item.setVideo(vo.getVideo());
                if (param.getType() == ContentPlatformGzhPlanTypeEnum.FWH_PUSH.getVal()) {
                    String response = touLiuHttpClient.sendAdFlowAddRequest(GET_SMALL_PAGE_URL, String.valueOf(vo.getVideoId()), "fwhdyy",
                            loginAccount.getChannel(), "自动", "公众号", "文章插小程序", "文字和小程序", account.getGhId());
                    JSONObject json = JSONObject.parseObject(response);
                    SmallPageUrlDetail smallPageUrlDetail = json.getJSONObject("data").toJavaObject(SmallPageUrlDetail.class);
                    item.setPageUrl(smallPageUrlDetail.getUrl());
                }
                item.setCreateAccountId(loginAccount.getId());
                item.setCreateTimestamp(System.currentTimeMillis());
                gzhPlanVideoMapper.insertSelective(item);
            }
        }
        if (CollectionUtils.isNotEmpty(param.getVideoList())) {
            saveChangeVideoLog(planId, existsVideoIds, param.getVideoList());
        }
    }

    private void saveChangeVideoLog(Long planId, List<Long> existsVideoIds, List<GzhPlanVideoContentItemParam> videoList) {
        List<Long> newVideoIds = videoList.stream().map(GzhPlanVideoContentItemParam::getVideoId).collect(Collectors.toList());
        boolean changed = false;
        for (Long newVideoId : newVideoIds) {
            if (!existsVideoIds.contains(newVideoId)) {
                changed = true;
                break;
            }
        }
        if (changed) {
            // 保存修改日志
            ContentPlatformGzhPlanChangeLog log = new ContentPlatformGzhPlanChangeLog();
            log.setPlanId(planId);
            log.setOldPlanIds(JSONObject.toJSONString(existsVideoIds));
            log.setNewPlanIds(JSONObject.toJSONString(newVideoIds));
            log.setCreateTimestamp(System.currentTimeMillis());
            gzhPlanChangeLogMapper.insertSelective(log);
        }
    }

    private List<ContentPlatformGzhPlanVideo> getGzhVideoByPlanId(Long planId) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdEqualTo(planId);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    private void updateGhDetail(ContentPlatformGzhAccount account, Integer selectVideoType, List<Long> videoIds) {
        GhDetail ghDetail = ghDetailService.getGhDetailByGhIdType(account.getGhId(), GhTypeEnum.THIRD_PARTY_GH.type);
        GhDetailVo detailVo = new GhDetailVo();
        Integer strategyStatus = selectVideoType == 0 ? StrategyStatusEnum.DEFAULT.status : StrategyStatusEnum.STRATEGY.status;
        if (Objects.nonNull(ghDetail)) {
            BeanUtils.copyProperties(ghDetail, detailVo);
            detailVo.setVideoIds(videoIds);
            detailVo.setStrategyStatus(strategyStatus);
            detailVo.setAutoreplySendMinigramNum(videoIds.size());
            detailVo.setCategory1(account.getContentType());
            ghDetailService.updateDetail(detailVo);
        } else {
            ContentPlatformAccount loginUser = LoginUserContext.getUser();
            detailVo.setAccountId(account.getGhId());
            detailVo.setAccountName(account.getName());
            detailVo.setType(GhTypeEnum.THIRD_PARTY_GH.type);
            detailVo.setChannel(loginUser.getChannel());
            detailVo.setCategory1(account.getContentType());
            detailVo.setStrategyStatus(strategyStatus);
            detailVo.setVideoIds(videoIds);
            detailVo.setAutoreplySendMinigramNum(videoIds.size());
            ghDetailService.addGhDetail(detailVo);
        }
    }

    @Override
    public List<ContentPlatformGzhPlan> getGzhPlanListByCooperateAccountId(Long accountId) {
        ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
        example.createCriteria().andAccountIdEqualTo(accountId).andStatusEqualTo(1);
        return gzhPlanMapper.selectByExample(example);
    }

    @Override
    public List<ContentPlatformGzhPlan> getGzhPlanListByCreateAccountId(Long createAccountId) {
        ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
        example.createCriteria().andCreateAccountIdEqualTo(createAccountId).andStatusEqualTo(1);
        return gzhPlanMapper.selectByExample(example);
    }

    @Override
    public List<ContentPlatformGzhPlan> getGzhPlanListByCooperateAccountIds(List<Long> accountIds) {
        ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
        example.createCriteria().andAccountIdIn(accountIds);
        return gzhPlanMapper.selectByExample(example);
    }

    @Override
    public Page<VideoContentItemVO> getVideoContentList(VideoContentListParam param) {
        Page<VideoContentItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        String dt = planMapperExt.getVideoMaxDt();
        int count = planMapperExt.getVideoCount(param, dt, videoMinScore);
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformVideo> videoList = planMapperExt.getVideoList(param, dt, videoMinScore, offset, param.getPageSize());
        List<VideoContentItemVO> list = buildVideoContentItemVOList(videoList);
        result.setObjs(list);
        return result;
    }

    private List<VideoContentItemVO> buildVideoContentItemVOList(List<ContentPlatformVideo> videoList) {
        if (CollectionUtils.isEmpty(videoList)) {
            return null;
        }
        List<Long> videoIds = videoList.stream()
                .filter(o -> !StringUtils.hasText(o.getCover()))
                .map(ContentPlatformVideo::getVideoId).distinct().collect(Collectors.toList());
        Map<Long, VideoDetail> coverMap = new HashMap<>();
        for (List<Long> partition : Lists.partition(videoIds, 20)) {
            Set<Long> ids = new HashSet<>(partition);
            coverMap.putAll(messageAttachmentService.getVideoDetail(ids));
        }
        List<VideoContentItemVO> result = new ArrayList<>();
        for (ContentPlatformVideo video : videoList) {
            VideoContentItemVO item = new VideoContentItemVO();
            item.setVideoId(video.getVideoId());
            item.setTitle(video.getTitle());
            item.setCover(video.getCover());
            VideoDetail detail = coverMap.get(video.getVideoId());
            if (Objects.nonNull(detail)) {
                String cover = detail.getCover().substring(0, detail.getCover().indexOf("/watermark"));
                item.setCover(cover);
            }
            item.setVideo(video.getVideo());
            item.setScore(video.getScore());
            result.add(item);
        }
        return result;
    }

    @Override
    public List<String> getVideoContentCategoryList() {
        String dt = planMapperExt.getVideoMaxDt();
        return planMapperExt.getVideoCategoryList(dt);
    }

    @Override
    public List<String> getVideoContentCoverFrameList(VideoContentCoverFrameParam param) {
        return managerApiService.getCoverImagePaths(param.getVideoId());
    }

    @Override
    public Page<QwPlanItemVO> qwPlanList(QwPlanListParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        Page<QwPlanItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = planMapperExt.getQwPlanCount(param, loginAccount.getId());
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformQwPlan> planList = planMapperExt.getQwPlanList(param,
                loginAccount.getId(), offset, param.getPageSize());
        List<QwPlanItemVO> list = buildQwPlanItemVOList(planList);
        result.setObjs(list);
        return result;
    }

    private List<QwPlanItemVO> buildQwPlanItemVOList(List<ContentPlatformQwPlan> planList) {
        if (CollectionUtils.isEmpty(planList)) {
            return null;
        }
        List<Long> planIds = planList.stream().map(ContentPlatformQwPlan::getId).collect(Collectors.toList());
        List<ContentPlatformQwPlanVideo> videoList = getQwPlanVideoList(planIds);
        Map<Long, ContentPlatformQwPlanVideo> videoMap = videoList.stream()
                .collect(Collectors.toMap(ContentPlatformQwPlanVideo::getPlanId, Function.identity()));
        List<QwPlanItemVO> result = new ArrayList<>();
        for (ContentPlatformQwPlan qwPlan : planList) {
            QwPlanItemVO planItemVO = new QwPlanItemVO();
            planItemVO.setId(qwPlan.getId());
            planItemVO.setType(qwPlan.getType());
            planItemVO.setScene(qwPlan.getScene());
            planItemVO.setPageUrl(qwPlan.getPageUrl());
            planItemVO.setCreateTimestamp(qwPlan.getCreateTimestamp());
            ContentPlatformQwPlanVideo videoItem = videoMap.get(qwPlan.getId());
            if (Objects.nonNull(videoItem)) {
                planItemVO.setTitle(videoItem.getTitle());
                planItemVO.setCover(videoItem.getCover());
                planItemVO.setVideo(videoItem.getVideo());
            }
            result.add(planItemVO);
        }
        return result;
    }

    @Override
    public List<ContentPlatformQwPlanVideo> getQwPlanVideoList(List<Long> planIds) {
        ContentPlatformQwPlanVideoExample example = new ContentPlatformQwPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return qwPlanVideoMapper.selectByExample(example);
    }

    @Override
    public List<ContentPlatformVideo> getVideoContentListByVideoIds(List<Long> videoIds) {
        ContentPlatformVideoExample example = new ContentPlatformVideoExample();
        example.createCriteria().andVideoIdIn(videoIds);
        return contentPlatformVideoMapper.selectByExample(example);
    }

    @Override
    public List<QwPlanItemVO> qwPlanSave(QwPlanSaveParam param) {
        ContentPlatformAccount loginUser = LoginUserContext.getUser();
        Long now = System.currentTimeMillis();
        List<ContentPlatformQwPlan> list = new ArrayList<>();
        for (QwPlanSaveVideoParam videoParam : param.getVideoList()) {
            ContentPlatformQwPlan qwPlan = new ContentPlatformQwPlan();
            qwPlan.setType(param.getType());
            qwPlan.setScene(videoParam.getScene());
            String pageUrl = messageAttachmentService.getPage(loginUser.getChannel(), "wxbdd2a2e93d9a6e25",
                    "dyyqw", "企微", QwPlanTypeEnum.from(param.getType()).getDescription(),
                    "位置1", videoParam.getVideoId());
            String rootSourceId = MessageUtil.getRootSourceId(pageUrl);
            qwPlan.setPageUrl(pageUrl);
            qwPlan.setRootSourceId(rootSourceId);
            qwPlan.setCreateAccountId(loginUser.getId());
            qwPlan.setCreateTimestamp(now);
            planMapperExt.insertQwPlanReturnId(qwPlan);
            list.add(qwPlan);
            // 保存视频内容
            saveQwPlanVideo(videoParam, qwPlan.getId(), loginUser.getId());
        }
        return buildQwPlanItemVOList(list);
    }

    @Override
    public void saveQwPlanVideo(QwPlanSaveVideoParam videoParam, Long planId, Long loginAccountId) {
        ContentPlatformQwPlanVideo item = new ContentPlatformQwPlanVideo();
        item.setPlanId(planId);
        item.setVideoId(videoParam.getVideoId());
        item.setTitle(videoParam.getTitle());
        item.setCover(videoParam.getCover());
        item.setVideo(videoParam.getVideo());
        item.setCreateAccountId(loginAccountId);
        item.setCreateTimestamp(System.currentTimeMillis());
        qwPlanVideoMapper.insertSelective(item);
    }

    @Override
    public String getSharePic(String pageUrl) {
        return vlogApiService.getSharePic(pageUrl, shareAppType);
    }

    @Override
    public List<ContentPlatformGzhPlanVideo> getGzhPlanVideoListByCooperateAccountId(String ghId) {
        return planMapperExt.getGzhPlanVideoListByCooperateAccountId(ghId);
    }

    @Override
    public List<ContentPlatformQwPlan> getQwPlanListByRootSourceIds(List<String> rootSourceIds) {
        ContentPlatformQwPlanExample example = new ContentPlatformQwPlanExample();
        example.createCriteria().andRootSourceIdIn(rootSourceIds);
        return qwPlanMapper.selectByExample(example);
    }

    @Override
    public List<ContentPlatformQwPlanVideo> getQwPlanVideoListByPlanIds(List<Long> planIds) {
        ContentPlatformQwPlanVideoExample example = new ContentPlatformQwPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return qwPlanVideoMapper.selectByExample(example);
    }

    @Override
    public String getGzhChannel(String ghId) {
        ContentPlatformGzhAccount gzhAccount = cooperateAccountService.getGzhAccount(ghId);
        if (Objects.nonNull(gzhAccount)) {
            ContentPlatformAccount account = accountService.getAccountById(gzhAccount.getCreateAccountId());
            ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
            example.createCriteria().andCreateAccountIdEqualTo(account.getId()).andAccountIdEqualTo(gzhAccount.getId())
                    .andStatusEqualTo(1).andTypeEqualTo(ContentPlatformGzhPlanTypeEnum.FWH_PUSH.getVal());
            List<ContentPlatformGzhPlan> list = gzhPlanMapper.selectByExample(example);
            if (CollectionUtils.isNotEmpty(list)) {
                return account.getChannel();
            }
        }
        return null;
    }

    @Override
    public void gzhPlanImport(List<String> channels) {
        List<ContentPlatformAccount> accountList = accountService.getAccountListByChannels(channels);
        for (ContentPlatformAccount account : accountList) {
            Long now = System.currentTimeMillis();
            List<GhDetail> ghDetailList = ghDetailService.getByChannel(account.getChannel());
            ghDetailList = ghDetailList.stream()
                    .filter(o -> Objects.equals(o.getStrategyStatus(), StrategyStatusEnum.STRATEGY.status))
                    .collect(Collectors.toList());
            if (CollectionUtils.isEmpty(ghDetailList)) {
                continue;
            }
            List<String> ghIds = ghDetailList.stream().map(GhDetail::getGhId).collect(Collectors.toList());
            List<ContentPlatformGzhAccount> gzhAccountList = cooperateAccountService.getCooperateAccountListByGhIds(account.getId(), ghIds);
            List<ContentPlatformGzhPlan> existsPlanList = getGzhPlanListByCreateAccountId(account.getId());
            List<Long> existsAccountIds = existsPlanList.stream().map(ContentPlatformGzhPlan::getAccountId).distinct().collect(Collectors.toList());
            Map<String, ContentPlatformGzhAccount> gzhAccountMap = gzhAccountList.stream()
                    .collect(Collectors.toMap(ContentPlatformGzhAccount::getGhId, Function.identity()));
            for (GhDetail ghDetail : ghDetailList) {
                ContentPlatformGzhAccount gzhAccount = gzhAccountMap.get(ghDetail.getGhId());
                if (Objects.isNull(gzhAccount)) {
                    continue;
                }
                ContentPlatformGzhPlan gzhPlan = new ContentPlatformGzhPlan();
                if (existsAccountIds.contains(gzhAccount.getId())) {
                    continue;
                }
                gzhPlan.setAccountId(gzhAccount.getId());
                gzhPlan.setType(ContentPlatformGzhPlanTypeEnum.AUTO_REPLY.getVal());
                gzhPlan.setScene(0);
                gzhPlan.setSelectVideoType(ghDetail.getStrategyStatus());
                gzhPlan.setPublishStage(PublishStageEnum.USER.getVal());
                if (StringUtils.hasText(gzhAccount.getExternalId())) {
                    if (aigcApiService.checkContainMessagePlan(gzhAccount.getExternalId())) {
                        aigcApiService.closeAccountMessagePublishPlan(gzhAccount.getExternalId());
                        gzhPlan.setPublishStage(PublishStageEnum.PLATFORM.getVal());
                        String externalId = aigcApiService.createPublishPlan(gzhAccount.getExternalId(), gzhAccount.getName(), account.getChannel());
                        gzhPlan.setExternalId(externalId);
                        gzhPlan.setStagePublishStatus(1);
                    }
                }
                gzhPlan.setCreateAccountId(account.getId());
                gzhPlan.setCreateTimestamp(now);
                gzhPlan.setUpdateTimestamp(now);
                planMapperExt.insertGzhPlanReturnId(gzhPlan);
                if (!StringUtils.hasText(ghDetail.getVideoIds())) {
                    continue;
                }
                List<Long> videoIds = JSONObject.parseArray(ghDetail.getVideoIds(), Long.class);
                if (CollectionUtils.isEmpty(videoIds)) {
                    continue;
                }
                Map<Long, VideoDetail> videoMap = messageAttachmentService.getVideoDetail(new HashSet<>(videoIds));
                for (Map.Entry<Long, VideoDetail> entry : videoMap.entrySet()) {
                    Long videoId = entry.getKey();
                    VideoDetail videoDetail = entry.getValue();
                    ContentPlatformGzhPlanVideo item = new ContentPlatformGzhPlanVideo();
                    item.setPlanId(gzhPlan.getId());
                    item.setVideoId(videoId);
                    item.setTitle(videoDetail.getTitle());
                    item.setCover(videoDetail.getCover());
                    item.setVideo(videoDetail.getVideoPath());
                    item.setCreateAccountId(account.getId());
                    item.setCreateTimestamp(System.currentTimeMillis());
                    gzhPlanVideoMapper.insertSelective(item);
                }
            }
        }
    }

    @Override
    public void qwPlanImport(List<String> channels) {
        List<AdPutFlowRecordTencent> list = new ArrayList<>();
        for (String channel : channels) {
            Integer count = adApiService.getPutFlowListTencentCount(channel, 1, 1);
            if (count == 0) {
                continue;
            }
            int pageNum = 1000;
            int totalPage = (count / pageNum) + 1;
            for (int i = 1; i <= totalPage; i++) {
                list.addAll(adApiService.getPutFlowListTencent(channel, i, pageNum));
            }
        }
        List<String> filterChannels = Arrays.asList("ml2025", "mj", "nh", "qc", "tczy", "wb", "xj", "wxm", "wxmx1", "wxmx", "xycsd", "xycsd1", "xycsd2", "xycsd3", "xycsd4", "xycsd5", "xycsd6", "xycsd7", "xycsd8", "xycsd9", "xycsd10", "xycsd11", "xycsd12", "xycsd13", "xycsd14", "xycsd15", "xycsd16", "xycsd17", "xycsd18", "shy");
        Map<String, List<AdPutFlowRecordTencent>> channelMap = list.stream()
                .filter(o -> filterChannels.contains(o.getChannel()))
                .collect(Collectors.groupingBy(AdPutFlowRecordTencent::getChannel));
        for (Map.Entry<String, List<AdPutFlowRecordTencent>> entry : channelMap.entrySet()) {
            String channel = entry.getKey();
            List<AdPutFlowRecordTencent> recordList = entry.getValue();
            ContentPlatformAccount account = accountService.getAccountByChannel(channel);
            List<String> rootSourceIds = recordList.stream().map(AdPutFlowRecordTencent::getRootSourceId).distinct().collect(Collectors.toList());
            List<ContentPlatformQwPlan> existsPlanList = getQwPlanListByRootSourceIds(rootSourceIds);
            List<String> existsRootSourceIds = existsPlanList.stream().map(ContentPlatformQwPlan::getRootSourceId).collect(Collectors.toList());
            List<Long> videoIds = recordList.stream().filter(o -> !existsRootSourceIds.contains(o.getRootSourceId()))
                    .map(AdPutFlowRecordTencent::getVideoId).distinct().collect(Collectors.toList());
            Map<Long, VideoDetail> videoMap = new HashMap<>();
            for (List<Long> partition : Lists.partition(videoIds, 20)) {
                Set<Long> ids = new HashSet<>(partition);
                videoMap.putAll(messageAttachmentService.getVideoDetail(ids));
            }
            Long now = System.currentTimeMillis();
            for (AdPutFlowRecordTencent record : recordList) {
                if (existsRootSourceIds.contains(record.getRootSourceId())) {
                    continue;
                }
                ContentPlatformQwPlan qwPlan = new ContentPlatformQwPlan();
                QwPlanTypeEnum typeEnum = QwPlanTypeEnum.from(record.getPutTypeTwo());
                qwPlan.setType(typeEnum.getVal());
                qwPlan.setScene(0);
                qwPlan.setPageUrl(record.getUrl());
                qwPlan.setRootSourceId(record.getRootSourceId());
                qwPlan.setCreateAccountId(account.getId());
                qwPlan.setCreateTimestamp(now);
                // 保存视频内容
                VideoDetail videoDetail = videoMap.get(record.getVideoId());
                if (Objects.isNull(videoDetail)) {
                    continue;
                }
                QwPlanSaveVideoParam videoParam = new QwPlanSaveVideoParam();
                videoParam.setVideoId(record.getVideoId());
                videoParam.setTitle(record.getVideoTitle());
                videoParam.setCover(videoDetail.getCover());
                videoParam.setVideo(videoDetail.getVideoPath());
                planMapperExt.insertQwPlanReturnId(qwPlan);
                saveQwPlanVideo(videoParam, qwPlan.getId(), account.getId());
            }
        }
    }

}
