package com.tzld.piaoquan.api.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.api.common.enums.ReplyStrategyServiceEnum;
import com.tzld.piaoquan.growth.common.dao.mapper.CgiReplyBucketDataMapper;
import com.tzld.piaoquan.growth.common.dao.mapper.GhDetailMapper;
import com.tzld.piaoquan.api.model.bo.BucketDataParam;
import com.tzld.piaoquan.api.model.bo.ReplyBucketData;
import com.tzld.piaoquan.growth.common.model.po.CgiReplyBucketData;
import com.tzld.piaoquan.growth.common.model.po.CgiReplyBucketDataExample;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.model.po.GhDetailExample;
import com.tzld.piaoquan.api.service.CgiReplyService;
import com.tzld.piaoquan.api.service.strategy.ReplyStrategyService;
import com.tzld.piaoquan.growth.common.utils.LarkRobotUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Service
public class CgiReplyServiceImpl implements CgiReplyService {

    @Autowired
    private ApplicationContext applicationContext;

    private Map<String, ReplyStrategyService> strategyServiceMap;

    @Autowired
    private GhDetailMapper ghDetailMapper;

    @Autowired
    private CgiReplyBucketDataMapper cgiReplyBucketDataMapper;

    @PostConstruct
    public void init() {
        strategyServiceMap = applicationContext.getBeansOfType(ReplyStrategyService.class);
    }


    @Override
    public ReplyBucketData getRgiReplyData(BucketDataParam bucketDataParam) {
        String ghId = bucketDataParam.getGhId();
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andGhIdEqualTo(ghId).andIsDeleteEqualTo(0);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(ghDetails)) {
            LarkRobotUtil.sendMessage("公众号信息查询失败 ghId=" + ghId);
            return null;
        }
        GhDetail ghDetail = ghDetails.get(0);
        bucketDataParam.setStrategyStatus(ghDetail.getStrategyStatus());
        bucketDataParam.setRate(ghDetail.getRate());
        bucketDataParam.setContentDetail(ghDetail.getContentDetail());
        if (StringUtils.isNotEmpty(ghDetail.getVideoIds())) {
            bucketDataParam.setVideos(JSONArray.parseArray(ghDetail.getVideoIds(), Long.class));
        }
        if (Objects.equals(ghDetail.getType(), GhTypeEnum.THIRD_PARTY_GH.type)) {
            bucketDataParam.setChannel(ghDetail.getChannel());
            return getPushMessageData(bucketDataParam);
        }
        return getRgiBucketData(bucketDataParam);
    }

    private ReplyBucketData getRgiBucketData(BucketDataParam bucketDataParam) {
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(ReplyStrategyServiceEnum.BUCKET_STRATEGY_V1)) {
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }

    private ReplyBucketData getPushMessageData(BucketDataParam bucketDataParam) {
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(ReplyStrategyServiceEnum.THIRD_PARTY_PUSH_MESSAGE_STRATEGY_V1)) {
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }

    @Override
    public List<CgiReplyBucketData> getCgiReplyBucketDataListByGhIds(List<String> ghIds, String strategy) {
        CgiReplyBucketDataExample example = new CgiReplyBucketDataExample();
        example.createCriteria().andGhIdIn(ghIds).andStrategyEqualTo(strategy).andIsDeleteEqualTo(0);
        return cgiReplyBucketDataMapper.selectByExample(example);
    }

    @Override
    public List<CgiReplyBucketData> getCgiReplyBucketDataListByVideoId(Long videoId) {
        CgiReplyBucketDataExample example = new CgiReplyBucketDataExample();
        example.createCriteria().andMiniVideoIdEqualTo(videoId).andIsDeleteEqualTo(0);
        return cgiReplyBucketDataMapper.selectByExample(example);
    }

    @Override
    public List<CgiReplyBucketData> getCgiReplyBucketDataListByGhIdVideoId(String ghId, List<Long> videoIds, String strategy) {
        CgiReplyBucketDataExample example = new CgiReplyBucketDataExample();
        example.createCriteria().andGhIdEqualTo(ghId).andMiniVideoIdIn(videoIds)
                .andStrategyEqualTo(strategy).andIsDeleteEqualTo(0);
        return cgiReplyBucketDataMapper.selectByExample(example);
    }
}
