package com.tzld.piaoquan.api.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.api.model.vo.GhDetailVo;
import com.tzld.piaoquan.api.service.GhDetailService;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.growth.common.common.enums.StrategyStatusEnum;
import com.tzld.piaoquan.growth.common.dao.mapper.GhDetailMapper;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.model.po.GhDetailExample;
import com.tzld.piaoquan.growth.common.utils.DateUtil;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Slf4j
@Service
public class GhDetailServiceImpl implements GhDetailService {

    @Autowired
    private GhDetailMapper ghDetailMapper;


    @Override
    public CommonResponse<Page<GhDetailVo>> getGhDetailList(Integer pageNum, Integer pageSize,
                                                            String accountId, String accountName) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (pageSize > 100) {
            pageSize = 100;
        }
        Page<GhDetailVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        GhDetailExample example = new GhDetailExample();
        example.setOrderByClause("create_time desc");
        GhDetailExample.Criteria criteria = example.createCriteria().andIsDeleteEqualTo(0);
        if (StringUtils.isNotEmpty(accountId)) {
            criteria.andGhIdEqualTo(accountId);
        }
        if (StringUtils.isNotEmpty(accountName)) {
            criteria.andGhNameLike("%" + accountName + "%");
        }
        example.setPage(page);
        long total = ghDetailMapper.countByExample(example);
        page.setTotalSize((int) total);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        List<GhDetailVo> ghDetailVos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(ghDetails)) {
            for (GhDetail ghDetail : ghDetails) {
                GhDetailVo ghDetailVo = new GhDetailVo();
                BeanUtils.copyProperties(ghDetail, ghDetailVo);
                ghDetailVo.setAccountId(ghDetail.getGhId());
                ghDetailVo.setAccountName(ghDetail.getGhName());
                ghDetailVo.setTypeName(GhTypeEnum.getTypeName(ghDetailVo.getType()));
                ghDetailVo.setCreateTime(DateUtil.getDateString(ghDetail.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
                ghDetailVo.setUpdateTime(DateUtil.getDateString(ghDetail.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
                ghDetailVo.setChannel(ghDetail.getChannel());
                if (StringUtils.isNotEmpty(ghDetail.getVideoIds())) {
                    List<Long> videoIds = JSONObject.parseArray(ghDetail.getVideoIds(), Long.class);
                    ghDetailVo.setVideoIds(videoIds);
                }
                ghDetailVo.setStrategyStatusName(StrategyStatusEnum.getStrategyStatusName(ghDetail.getStrategyStatus()));
                ghDetailVos.add(ghDetailVo);
            }
        }
        page.setObjs(ghDetailVos);
        return CommonResponse.success(page);
    }


    @Override
    public CommonResponse<Void> addGhDetail(GhDetailVo ghDetailVo) {
        if (ghDetailVo == null
                || StringUtils.isEmpty(ghDetailVo.getAccountId())
                || StringUtils.isEmpty(ghDetailVo.getAccountName())
                || ghDetailVo.getType() == null
                || StringUtils.isEmpty(ghDetailVo.getCategory1())) {
            return CommonResponse.create(500, "参数错误");
        }

        CommonResponse<Void> checkParamRes = checkParam(ghDetailVo);
        if (checkParamRes != null) {
            return checkParamRes;
        }

        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountId())) {
                ghDetail.setGhId(ghDetailVo.getAccountId().trim());
            }
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountName())) {
                ghDetail.setGhName(ghDetailVo.getAccountName());
            }
            ghDetail.setGhId(ghDetailVo.getAccountId());
            ghDetail.setGhName(ghDetailVo.getAccountName());
            if (!CollectionUtils.isEmpty(ghDetailVo.getVideoIds())) {
                ghDetail.setVideoIds(JSONObject.toJSONString(ghDetailVo.getVideoIds()));
            }
            if (StringUtils.isEmpty(ghDetail.getContentDetail())) {
                ghDetail.setContentDetail(null);
            }
            ghDetailMapper.insertSelective(ghDetail);
            return CommonResponse.success();
        } catch (Exception e) {
            log.error("addGhDetail error", e);
        }
        return CommonResponse.create(500, "插入失败");
    }

    @Override
    public CommonResponse<Void> updateDetail(GhDetailVo ghDetailVo) {
        if (ghDetailVo == null || ghDetailVo.getId() == null) {
            return CommonResponse.create(500, "参数错误");
        }
        CommonResponse<Void> checkParamRes = checkParam(ghDetailVo);
        if (checkParamRes != null) {
            return checkParamRes;
        }
        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountId())) {
                ghDetail.setGhId(ghDetailVo.getAccountId().trim());
            }
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountName())) {
                ghDetail.setGhName(ghDetailVo.getAccountName());
            }
            if (StringUtils.isEmpty(ghDetail.getContentDetail())) {
                ghDetail.setContentDetail(null);
            }
            ghDetail.setGhId(ghDetailVo.getAccountId());
            ghDetail.setGhName(ghDetailVo.getAccountName());
            ghDetail.setVideoIds(JSONObject.toJSONString(ghDetailVo.getVideoIds()));
            ghDetailMapper.updateByPrimaryKeySelective(ghDetail);

            if (StringUtils.isEmpty(ghDetail.getContentDetail()) || StringUtils.isEmpty(ghDetail.getRate())) {
                GhDetail updateGhDetail = ghDetailMapper.selectByPrimaryKey(ghDetail.getId());
                if (StringUtils.isEmpty(ghDetail.getContentDetail())) {
                    updateGhDetail.setContentDetail(null);
                }
                if (StringUtils.isEmpty(ghDetail.getRate())) {
                    updateGhDetail.setRate(null);
                }
                updateGhDetail.setUpdateTime(new Date());
                ghDetailMapper.updateByPrimaryKey(updateGhDetail);
            }


            return CommonResponse.success();
        } catch (Exception e) {
            log.error("updateDetail error", e);
        }
        return CommonResponse.create(500, "修改失败");
    }


    @Override
    public CommonResponse<Void> deleteDetail(Long id) {
        GhDetail ghDetail = new GhDetail();
        ghDetail.setId(id);
        ghDetail.setIsDelete(1);
        ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        return CommonResponse.success();
    }

    private CommonResponse<Void> checkParam(GhDetailVo ghDetailVo) {
        if (Objects.equals(ghDetailVo.getType(), GhTypeEnum.THIRD_PARTY_GH.type)) {
            if (StringUtils.isEmpty(ghDetailVo.getChannel())) {
                return CommonResponse.create(500, "渠道不能为空");
            }
        }
        if (StringUtils.isNotEmpty(ghDetailVo.getRate())) {
            if (!JSONArray.isValidArray(ghDetailVo.getRate())) {
                return CommonResponse.create(500, "保底比率格式错误");
            }
            List<Integer> list = JSONArray.parseArray(ghDetailVo.getRate(), Integer.class);
            if (!CollectionUtils.isEmpty(list)) {
                for (Integer i : list) {
                    if (i < 0 || i > 9) {
                        return CommonResponse.create(500, "保底比率需要再0-9之间");
                    }
                }
            }
        }

        if (StringUtils.isNotEmpty(ghDetailVo.getContentDetail())) {
            ghDetailVo.setContentDetail(ghDetailVo.getContentDetail().trim());
            if (!JSONObject.isValidObject(ghDetailVo.getContentDetail())) {
                return CommonResponse.create(500, "文章内容格式错误");
            }
            JSONObject contentDetail = JSONObject.parseObject(ghDetailVo.getContentDetail());
            if (StringUtils.isEmpty(contentDetail.getString("coverUrl"))) {
                return CommonResponse.create(500, "文章封面信息错误");
            }
            if (StringUtils.isEmpty(contentDetail.getString("newsDescription"))) {
                return CommonResponse.create(500, "文章内容错误");
            }
            if (StringUtils.isEmpty(contentDetail.getString("newsUrl"))) {
                return CommonResponse.create(500, "文章路径错误");
            }
            if (StringUtils.isEmpty(contentDetail.getString("title"))) {
                return CommonResponse.create(500, "文章标题错误");
            }
        }
        return null;
    }

    @Override
    public GhDetail getGhDetailByGhIdType(String ghId, Integer type) {
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andGhIdEqualTo(ghId).andTypeEqualTo(type).andIsDeleteEqualTo(0);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(ghDetails)) {
            return ghDetails.get(0);
        }
        return null;
    }

    @Override
    public List<GhDetail> getByChannel(String channel) {
        GhDetailExample ghDetailExample = new GhDetailExample();
        ghDetailExample.createCriteria().andChannelEqualTo(channel).andIsDeleteEqualTo(0);
        return ghDetailMapper.selectByExample(ghDetailExample);
    }


}
